@@ -7,7 +7,7 @@ use std::{
 use bit_iter::BitIter;
 use itertools::Itertools;
 
-use crate::{pyo3_prelude::*, util::BitMask64};
+use crate::{pyo3_prelude::*, util::BitMask128};
 
 use super::{
     computational_node::EinsumArgs, deep_map_unwrap, evaluate_fn, Add, Circuit,
@@ -1318,7 +1318,7 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                     )
                     .unwrap()
                     .extract(py)
-                    .unwrap()
+                    .ok()
             });
             python_result.map(|(index, spec)| {
                 let new_index = Index::try_new(inner.node.clone(), index, node.name_cloned())
@@ -1455,15 +1455,15 @@ fn basic_check_same_concat_shape() {
 }
 
 #[inline]
-pub fn is_not_outer(a: BitMask64, b: BitMask64) -> bool {
+pub fn is_not_outer(a: BitMask128, b: BitMask128) -> bool {
     let union = a & b;
     union == a || union == b
 }
 
 /// this takes bitmasks of axes and produces bitmasks of indices into argument bitmasks that each don't have any outer products
-pub fn bitmask_outer_product_sets(bitmasks: &Vec<BitMask64>) -> Vec<BitMask64> {
-    let mut result: Vec<BitMask64> = vec![];
-    let mut result_axes: Vec<BitMask64> = vec![];
+pub fn bitmask_outer_product_sets(bitmasks: &Vec<BitMask128>) -> Vec<BitMask128> {
+    let mut result: Vec<BitMask128> = vec![];
+    let mut result_axes: Vec<BitMask128> = vec![];
     for (input_idx, bm) in bitmasks.iter().enumerate() {
         let mut did_anything = false;
         let mut result_index_out: usize = 0;
@@ -1487,7 +1487,7 @@ pub fn bitmask_outer_product_sets(bitmasks: &Vec<BitMask64>) -> Vec<BitMask64> {
 
 #[test]
 fn test_bitmask_outer_product_sets() {
-    let ex = vec![1 + 4, 2 + 4, 4, 1];
+    let ex: Vec<u128> = vec![1 + 4, 2 + 4, 4, 1];
     dbg!(&ex);
     dbg!(bitmask_outer_product_sets(&ex));
     dbg!(bitmask_outer_product_sets(&ex)
@@ -1498,11 +1498,11 @@ fn test_bitmask_outer_product_sets() {
 
 #[pyfunction]
 pub fn add_outer_product_broadcasts_on_top(add: &Add) -> Option<Add> {
-    let axis_sets: Vec<BitMask64> = add
+    let axis_sets: Vec<BitMask128> = add
         .nodes_and_rank_differences()
         .iter()
         .map(|(child, rank_dif)| {
-            let mut bitmask: BitMask64 = 0;
+            let mut bitmask: BitMask128 = 0;
             for (i, l) in child.info().shape.iter().enumerate() {
                 if *l != 1 {
                     bitmask |= 1 << (rank_dif + i)
