@@ -1,4 +1,4 @@
-use crate::pyo3_prelude::*;
+use crate::{circuit::print::rust_expression_notation_circuit, pyo3_prelude::*};
 
 use super::{
     algebraic_rewrite::deep_optimize_einsums,
@@ -46,6 +46,8 @@ pub struct OptimizationSettings {
     pub numerical_scale_min: f64,
     #[pyo3(get, set)]
     pub numerical_scale_max: f64,
+    #[pyo3(get, set)]
+    pub capture_and_print: bool,
 }
 
 impl Default for OptimizationSettings {
@@ -60,6 +62,7 @@ impl Default for OptimizationSettings {
             adjust_numerical_scale: false,
             numerical_scale_min: 1e-8,
             numerical_scale_max: 1e8,
+            capture_and_print: false,
         }
     }
 }
@@ -76,7 +79,8 @@ impl OptimizationSettings {
         scheduling_simplify = "OptimizationSettings::default().scheduling_simplify",
         adjust_numerical_scale = "OptimizationSettings::default().adjust_numerical_scale",
         numerical_scale_min = "OptimizationSettings::default().numerical_scale_min",
-        numerical_scale_max = "OptimizationSettings::default().numerical_scale_max"
+        numerical_scale_max = "OptimizationSettings::default().numerical_scale_max",
+        capture_and_print = "OptimizationSettings::default().capture_and_print"
     )]
     fn new(
         verbose: usize,
@@ -88,6 +92,7 @@ impl OptimizationSettings {
         adjust_numerical_scale: bool,
         numerical_scale_min: f64,
         numerical_scale_max: f64,
+        capture_and_print: bool,
     ) -> Self {
         Self {
             verbose,
@@ -99,6 +104,7 @@ impl OptimizationSettings {
             adjust_numerical_scale,
             numerical_scale_min,
             numerical_scale_max,
+            capture_and_print,
         }
     }
 }
@@ -152,6 +158,9 @@ pub fn optimize_and_evaluate(
     if verbose > 0 {
         println!("Optimizing verbose {}", verbose)
     }
+    if settings.capture_and_print {
+        println!("{}", rust_expression_notation_circuit(circuit.clone()));
+    }
     let optimized_circuit = timed!(optimize_circuit(circuit, settings), 10, verbose >= 1);
     // return evaluate_fn(&*optimized_circuit).unwrap();
     let schedule = if settings.scheduling_naive {
