@@ -787,7 +787,16 @@ impl Index {
             ]),
             _ => {}
         }
-
+        let index = TensorIndex(
+            index
+                .0
+                .into_iter()
+                .map(|x| match x {
+                    TensorAxisIndex::Tensor(tensor) => TensorAxisIndex::Tensor(tensor.hashed()),
+                    _ => x,
+                })
+                .collect(),
+        );
         let mut out = Self {
             node,
             index,
@@ -862,16 +871,19 @@ impl CircuitNode for Index {
         })
     }
 
-    fn device_dtype_extra<'a>(&'a self) -> Box<dyn Iterator<Item = TorchDeviceDtypeOp> + 'a> {
-        Box::new(self.index.0.iter().filter_map(|x| match x {
-            TensorAxisIndex::Tensor(tensor) if !tensor.shape().is_empty() => {
-                let mut out: TorchDeviceDtypeOp = TorchDeviceDtype::from_tensor(tensor).into();
-                out.dtype = None;
-                Some(out)
-            }
-            _ => None,
-        }))
-    }
+    // commented this out bc cast_circuit is broken with this bc deep_map assumes you can construct
+    // original with new children
+    //
+    // fn device_dtype_extra<'a>(&'a self) -> Box<dyn Iterator<Item = TorchDeviceDtypeOp> + 'a> {
+    //     Box::new(self.index.0.iter().filter_map(|x| match x {
+    //         TensorAxisIndex::Tensor(tensor) if !tensor.shape().is_empty() => {
+    //             let mut out: TorchDeviceDtypeOp = TorchDeviceDtype::from_tensor(tensor).into();
+    //             out.dtype = None;
+    //             Some(out)
+    //         }
+    //         _ => None,
+    //     }))
+    // }
 }
 
 impl CircuitNodeAutoName for Index {
