@@ -106,41 +106,37 @@ impl ArrayConstant {
     }
 
     #[staticmethod]
-    #[args(args = "*")]
-    pub fn randn(args: Vec<usize>) -> Self {
+    pub fn randn_named(shape: Vec<usize>, name: Option<String>) -> Self {
         Python::with_gil(|py| {
             ArrayConstant::new(
                 PY_TORCH
                     .getattr(py, "randn")
                     .unwrap()
-                    .call(py, PyTuple::new(py, args), None)
+                    .call(py, (PyTuple::new(py, shape),), None)
                     .unwrap()
                     .extract(py)
                     .unwrap(),
-                None,
+                name,
             )
         })
     }
 
     #[staticmethod]
-    pub fn randn_seeded(shape: Vec<usize>, seed: usize) -> Self {
+    #[args(args = "*")]
+    pub fn randn(args: Vec<usize>) -> Self {
+        ArrayConstant::randn_named(args, None)
+    }
+
+    #[staticmethod]
+    pub fn randn_named_seeded(shape: Vec<usize>, name: Option<String>, seed: usize) -> Self {
         Python::with_gil(|py| {
             PY_TORCH
                 .getattr(py, "manual_seed")
                 .unwrap()
                 .call(py, (seed,), None)
                 .unwrap();
-            ArrayConstant::new(
-                PY_TORCH
-                    .getattr(py, "randn")
-                    .unwrap()
-                    .call(py, PyTuple::new(py, shape), None)
-                    .unwrap()
-                    .extract(py)
-                    .unwrap(),
-                None,
-            )
-        })
+        });
+        ArrayConstant::randn_named(shape, name)
     }
 
     #[staticmethod]
