@@ -1,10 +1,6 @@
 use crate::pyo3_prelude::*;
 use std::{collections::HashMap, iter::zip};
 
-use cached::proc_macro::cached;
-use pyo3::types::PyDict;
-use pyo3::types::PyTuple;
-
 use super::circuit_optimizer::optimize_and_evaluate;
 use super::{
     Add, ArrayConstant, CachedCircuitInfo, Circuit, CircuitConstructionError, CircuitNode,
@@ -13,10 +9,15 @@ use super::{
 use crate::py_types::{GENERALFUNCTIONS, PY_NONE};
 use crate::{
     circuit::PyCircuitBase,
-    circuit_node_auto_impl, circuit_node_extra_impl,
+    circuit_node_auto_impl, circuit_node_extra_impl, new_rc_unwrap,
     py_types::py_address,
     tensor_util::{Shape, TorchDeviceDtype},
 };
+use cached::proc_macro::cached;
+use lazy_static::lazy_static;
+use macro_rules_attribute::apply;
+use pyo3::types::PyDict;
+use pyo3::types::PyTuple;
 use uuid::uuid;
 
 #[pyo3::prelude::pyclass] // this needs to be sent to python all the time
@@ -53,7 +54,7 @@ impl PyGFSpecShapeGetter {
 macro_rules! gfspecs_wrap {
     ($py:expr,$name:literal,$non_batchable:literal) => {
         (
-            $name,
+            $name.to_owned(),
             GeneralFunctionSpec {
                 function: GENERALFUNCTIONS[$name].clone(),
                 get_shape: PyGFSpecShapeGetter {
@@ -64,29 +65,31 @@ macro_rules! gfspecs_wrap {
                 num_non_batchable_output_dims: $non_batchable,
                 input_batchability: vec![true],
                 name: $name.to_owned(),
+                is_official: true,
             },
         )
     };
 }
 
-#[cached]
-fn get_gfspec(name: String) -> Option<GeneralFunctionSpec> {
-    let map = Python::with_gil(|py| {
-        HashMap::from([
-            gfspecs_wrap!(py, "sigmoid", 0),
-            gfspecs_wrap!(py, "rsqrt", 0),
-            gfspecs_wrap!(py, "gelu", 0),
-            gfspecs_wrap!(py, "relu", 0),
-            gfspecs_wrap!(py, "softmax", 1),
-            gfspecs_wrap!(py, "log_softmax", 1),
-            gfspecs_wrap!(py, "log_exp_p_1", 0),
-            gfspecs_wrap!(py, "gaussian_pdf", 0),
-            gfspecs_wrap!(py, "gaussian_cdf", 0),
-        ])
-    });
-    map.get(&*name).cloned()
+lazy_static! {
+    static ref GENERAL_FUNCTIONS_SPECS: HashMap<String, GeneralFunctionSpec> =
+        Python::with_gil(|py| {
+            HashMap::from([
+                gfspecs_wrap!(py, "sigmoid", 0),
+                gfspecs_wrap!(py, "tanh", 0),
+                gfspecs_wrap!(py, "rsqrt", 0),
+                gfspecs_wrap!(py, "gelu", 0),
+                gfspecs_wrap!(py, "relu", 0),
+                gfspecs_wrap!(py, "reciprocal", 0),
+                gfspecs_wrap!(py, "log_exp_p_1", 0),
+                gfspecs_wrap!(py, "gaussian_pdf", 0),
+                gfspecs_wrap!(py, "gaussian_cdf", 0),
+                gfspecs_wrap!(py, "softmax", 1),
+                gfspecs_wrap!(py, "log_softmax", 1),
+                gfspecs_wrap!(py, "q_from_qr", 2),
+            ])
+        });
 }
-
 /// GeneralFunctionSpec contains all needed info about function, and is the same on all instances with the same function
 /// how batchability works: input_batchability is a mask indicating which inputs support batching. if none do, there is no batching.
 /// the number of non batchable dims in output, starting from end, is num_non_batchable_output_dims.
@@ -105,6 +108,8 @@ pub struct GeneralFunctionSpec {
     pub input_batchability: Vec<bool>,
     #[pyo3(get)]
     pub name: String,
+    #[pyo3(get)]
+    pub is_official: bool,
 }
 
 impl GeneralFunctionSpec {
@@ -151,6 +156,7 @@ impl GeneralFunctionSpec {
             num_non_batchable_output_dims,
             input_batchability,
             name,
+            is_official: false,
         }
     }
     pub fn is_batchable(&self) -> bool {
@@ -170,11 +176,12 @@ pub struct GeneralFunction {
 }
 
 impl GeneralFunction {
+    #[apply(new_rc_unwrap)]
     pub fn try_new(
         nodes: Vec<CircuitRc>,
         spec: GeneralFunctionSpec,
         name: Option<String>,
-    ) -> Result<Self, CircuitConstructionError> {
+    ) -> (Result<Self, CircuitConstructionError>) {
         let mut out = Self {
             nodes,
             spec,
@@ -226,7 +233,13 @@ impl CircuitNode for GeneralFunction {
         for node in &self.nodes {
             hasher.update(&node.info().hash);
         }
-        hasher.update(&self.spec.compute_hash());
+        if self.spec.is_official {
+            hasher.update(uuid!("2b355376-2e18-479e-8a52-1903ba3fd145").as_bytes());
+            hasher.update(self.spec.name.as_bytes());
+        } else {
+            hasher.update(uuid!("2f45e4d7-8019-43df-be98-23425312691b").as_bytes());
+            hasher.update(&self.spec.compute_hash());
+        }
         hasher
     }
 
@@ -326,9 +339,9 @@ impl GeneralFunction {
         spec_name: String,
         name: Option<String>,
     ) -> Result<Self, CircuitConstructionError> {
-        let spec = get_gfspec(spec_name.clone());
+        let spec = GENERAL_FUNCTIONS_SPECS.get(&spec_name.clone());
         let spec = spec.ok_or(CircuitConstructionError::UnknownGeneralFunction { spec_name })?;
-        GeneralFunction::try_new(nodes, spec, name)
+        GeneralFunction::try_new(nodes, spec.clone(), name)
     }
 }
 
