@@ -29,6 +29,7 @@ pub mod generalfunction;
 pub mod named_axes;
 pub mod nrc;
 pub mod print;
+mod repr;
 pub mod scatter_rewrite;
 pub mod scheduled_execution;
 pub mod scheduling_z3;
@@ -36,8 +37,8 @@ pub mod scheduling_z3;
 use crate::{
     cached_lambda, caching_struct,
     py_types::{PyShape, Tensor, CIRCUIT_RUST_TO_PY},
-    rearrange_spec::RearrangeSpec,
-    tensor_util::{Shape, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp},
+    rearrange_spec::{OpSize, RearrangeSpec},
+    tensor_util::{Shape, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp},
     util::AsOp,
 };
 pub use computational_node::{
@@ -1054,6 +1055,11 @@ impl PyCircuitBase {
         use_rust_comp(&self.0, &object.0, comp_op)
     }
 
+    #[getter]
+    fn hash(&self) -> [u8; 32] {
+        self.info().hash
+    }
+
     fn __hash__(&self) -> u64 {
         u64::from_le_bytes(
             self.info().hash[..std::mem::size_of::<u64>()]
