@@ -42,6 +42,7 @@ use pyo3::exceptions::PyValueError;
 
 /// reexport
 pub use pyo3;
+pub use uuid;
 
 #[cfg(not(feature = "real-pyo3"))]
 mod pyo3_prelude {
@@ -69,6 +70,7 @@ mod pyo3_prelude {
 
 use crate::pyo3_prelude::*;
 
+pub mod all_imports;
 mod caching;
 pub mod circuit;
 pub mod opt_einsum;
@@ -198,6 +200,15 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         circuit::algebraic_rewrite::add_outer_product_broadcasts_on_top,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::circuit_utils::replace_all_randn_seeded,
+        m
+    )?)?;
+
+    m.add_function(wrap_pyfunction!(
+        circuit::print::rust_expression_notation_circuit,
+        m
+    )?)?;
 
     // add more py function as desired or whatever...
     Ok(())
