@@ -2,7 +2,7 @@ use crate::pyo3_prelude::*;
 use crate::{
     circuit::{CircuitConstructionError, HashBytes},
     filter_by_variant,
-    py_types::{Tensor, CAST_TENSOR, PY_CAST_INT, PY_ID},
+    py_types::{Tensor, CAST_TENSOR, PY_CAST_INT, PY_ID, PY_TORCH},
     pycall,
     rearrange_spec::RearrangeSpec,
 };
@@ -308,6 +308,27 @@ impl TensorAxisIndex {
         }
         false
     }
+
+    pub fn new_tensor_randint_seeded(length: usize, base_length: usize, seed: usize) -> Self {
+        TensorAxisIndex::Tensor(pyo3::Python::with_gil(|py| {
+            PY_TORCH
+                .getattr(py, "manual_seed")
+                .unwrap()
+                .call(py, (seed,), None)
+                .unwrap();
+            PY_TORCH
+                .getattr(py, "randint")
+                .unwrap()
+                .call(
+                    py,
+                    (0, base_length, pyo3::types::PyTuple::new(py, vec![length])),
+                    None,
+                )
+                .unwrap()
+                .extract(py)
+                .unwrap()
+        }))
+    }
 }
 
 impl IntoPy<PyObject> for TensorAxisIndex {
@@ -433,6 +454,10 @@ impl TensorIndex {
         )
     }
 
+    pub fn ident(rank: usize) -> Self {
+        Self(vec![TensorAxisIndex::IDENT; rank])
+    }
+
     pub fn compute_hash(&self) -> HashBytes {
         let mut hasher = blake3::Hasher::new();
         for axis in self.0.iter() {
@@ -442,14 +467,7 @@ impl TensorIndex {
                     hasher.update(&i.to_le_bytes());
                 }
                 TensorAxisIndex::Tensor(t) => {
-                    Python::with_gil(|py| {
-                        let id: usize = PY_ID
-                            .call(py, (t.tensor(),), None)
-                            .unwrap()
-                            .extract(py)
-                            .unwrap();
-                        hasher.update(&id.to_le_bytes());
-                    });
+                    hasher.update(t.hashed().hash().unwrap());
                 } // dont bother to check tensor, if you want that canon first
                 TensorAxisIndex::Slice(sl) => sl.update_hash(&mut hasher),
             }
